import { createTheme, Palette, PaletteColor, PaletteKeys } from '@mui/material/styles';

import { ScreenSize } from '../screen.config';
import { FoundationTheme } from '../themes';
import nuskinTheme from '../themes/genome-foundation/';
import { isValidPaletteColor } from './common-utils';

export const generatePalette = (colors?: Palette, prefersDarkMode = false) => {
    if (typeof colors !== 'object' || !colors) {
        return {};
    }
    const allColorKeys = Object.keys(colors) as FoundationTheme.PaletteColors[];
    const colorKeys = allColorKeys?.filter((colorKey) => isValidPaletteColor(colorKey));
    return colorKeys.reduce(
        (result, paletteName: PaletteKeys) => {
            const colorObject = colors?.[paletteName] as PaletteColor;
            if (prefersDarkMode) {
                const darkMode = colorObject?.darkMode;
                if (darkMode) {
                    result[paletteName] = darkMode;
                }
            } else if (colorObject) {
                result[paletteName] = colorObject;
            }
            return result;
        },
        {} as Record<string, PaletteColor>,
    );
};

export const createNuskinTheme = (
    theme: FoundationTheme.Theme = nuskinTheme,
    prefersDarkMode?: boolean,
    locale?: string,
) => {
    const muiTheme = createTheme({ ...theme });
    const mode: FoundationTheme.Mode = prefersDarkMode ? 'dark' : 'light';
    const localeThemeFonts = theme?.fonts?.[locale || 'en-US'];

    const themeTypography = localeThemeFonts
        ? {
            button: {
                fontFamily: localeThemeFonts?.section?.buttons,
            },
            h1: {
                fontFamily: localeThemeFonts?.section?.headings,
            },
            h2: {
                fontFamily: localeThemeFonts?.section?.headings,
            },
            h3: {
                fontFamily: localeThemeFonts?.section?.headings,
            },
            h4: {
                fontFamily: localeThemeFonts?.section?.headings,
            },
            h5: {
                fontFamily: localeThemeFonts?.section?.headings,
            },
            h6: {
                fontFamily: localeThemeFonts?.section?.headings,
            },
            body1: {
                fontFamily: localeThemeFonts?.section?.body,
            },
            body2: {
                fontFamily: localeThemeFonts?.section?.body,
            },
            caption: {
                fontFamily: localeThemeFonts?.section?.captions,
            },
            subtitle1: {
                fontFamily: localeThemeFonts?.section?.subtitles,
            },
            subtitle2: {
                fontFamily: localeThemeFonts?.section?.subtitles,
            },
        }
        : {};
    return {
        ...muiTheme,
        palette: {
            ...muiTheme.palette,
            ...theme?.palette,
            mode,
        },
        fonts: localeThemeFonts,
        /*
         * Typography is just for the MUI components when render by default
         * NsTypography in foundation-ui-components will take over the line height and spacing
         */
        typography: {
            ...muiTheme.typography,
            ...themeTypography,
        },
        breakpoints: {
            values: ScreenSize,
        },
    };
};

export const iterateNuskinThemeColors = (colorObj: FoundationTheme.Colors | Record<string, string> = {}) => {
    const allThemeColors: Record<string, string | FoundationTheme.MainDarkColors> = {};

    const iterate = (
        colorObj: FoundationTheme.Colors | Record<string, string | FoundationTheme.MainDarkColors>,
        parentColor?: string,
    ) => {
        Object.keys(colorObj).forEach((key) => {
            const colorKey = parentColor ? `${parentColor}-${key}` : key;
            const colorValue = colorObj[key as FoundationTheme.PaletteColors];
            if (typeof colorValue === 'string') {
                allThemeColors[colorKey] = colorValue;
            } else {
                if (colorValue?.main) {
                    allThemeColors[colorKey] = colorValue as FoundationTheme.MainDarkColors;
                } else if (typeof colorValue === 'object') {
                    iterate(colorValue, colorKey);
                }
            }
        });
    };

    iterate(colorObj);
    return allThemeColors;
};

const generateColors = (name: string, value: string | { main?: string; dark?: string }) => {
    const makeColorCss = (colorValue: string) => `
        .bg-${name} {
            background: ${colorValue};
        }
        .text-${name} {
            color: ${colorValue};
        }
        .border-${name} {
            border-color: ${colorValue};
        }
    `;
    if (typeof value !== 'string' && value?.main) {
        let cssText = makeColorCss(value.main);
        if (value?.dark) {
            cssText += `
                .dark {
                    ${makeColorCss(value?.dark)}
                }
            `;
        }
        return cssText;
    }
    return makeColorCss(value as string);
};

export const generateGlobalColorCss = (theme?: FoundationTheme.Theme) => {
    if (!theme) {
        return '';
    }

    let colorCssString = '';
    /*
     * Nuskin Theme consists of palette, which is sent to Mui
     * The Nuskin Theme also has colors which is used to generate the helper css
     */
    const palette = theme?.palette;

    const themeColors = iterateNuskinThemeColors(theme?.colors);

    const paletteColorNames = Object.keys(palette || {}) as FoundationTheme.PaletteColors[];

    const paletteColors = paletteColorNames?.filter((color: FoundationTheme.PaletteColors) => {
        const colorInPalette = palette?.[color] as FoundationTheme.MainDarkColors;
        return colorInPalette?.main;
    });

    colorCssString = paletteColors?.reduce((result, colorName) => {
        const colorResult = palette?.[colorName] as FoundationTheme.MainDarkColors;
        const colorValue = colorResult?.main;
        if (colorValue) {
            result += generateColors(colorName, colorValue);
        }
        return result;
    }, '');

    colorCssString = Object.keys(themeColors)?.reduce((result, colorName: string) => {
        const colorValue = themeColors?.[colorName];
        if (colorValue) {
            result += generateColors(colorName, colorValue);
        }
        return result;
    }, colorCssString);

    return colorCssString;
};

export const generateTextAlignCss = () => {
    return `
        .text-left {
            text-align: left;
        }
        .text-center {
            text-align: center;
        }
        .text-right {
            text-align: right;
        }
        .text-justify {
            text-align: justify;
        }
        .text-start {
            text-align: start;
        }
        .text-end {
            text-align: end;
        }
    `;
};

export const generateFontFamilyCssClass = (fontSections?: Record<FoundationTheme.FontSections, string>) => {
    const sections = Object.keys(fontSections || {}) as FoundationTheme.FontSections[];
    if (sections?.length) {
        return sections.reduce((result, section) => {
            const fontFamily = fontSections?.[section];
            if (fontFamily) {
                result += `
                    .font-${section} {
                        font-family: ${fontFamily} !important;
                    }
                `;
            }
            return result;
        }, '');
    }
    return '';
};

